import logging
import click
from datetime import datetime

from vcftoolbox import get_vcf_handle

from loqusdb.utils import get_family
from loqusdb.vcf_tools import get_formated_variant
from loqusdb.exceptions import CaseError
from . import base_command

logger = logging.getLogger(__name__)

@base_command.command()
@click.argument('variant_file',
                    nargs=1,
                    type=click.Path(exists=True),
                    metavar='<vcf_file> or -'
)
@click.option('-f', '--family_file',
                    nargs=1, 
                    type=click.File('r'),
                    metavar='<ped_file>'
)
@click.option('-t' ,'--family_type', 
                type=click.Choice(['ped', 'alt', 'cmms', 'mip']), 
                default='ped',
                help='If the analysis use one of the known setups, please specify which one.'
)
@click.option('-i', '--family_id',
                    nargs=1, 
                    type=str,
                    help='The id for the case to add'
)
@click.pass_context
def delete(ctx, variant_file, family_file, family_type, family_id):
    """Delete the variants of a case from the frequency database.
    
    """
    if not family_file or family_id:
        logger.error("Please provide a family file or a case id")
        logger.info("Exiting")
        ctx.abort()
    
    family = get_family(
        family_lines=family_file,
        family_type=family_type
    )
    
    family_id = family.family_id
    
    adapter = ctx.obj['adapter']
    
    case = {
        'case_id': family_id,
    }
    
    try:
        adapter.delete_case(case)
    except CaseError as err:
        logger.warning(err.message)
        ctx.abort()
    
    affected_individuals = family.affected_individuals
    
    if variant_file == '-':
        logger.info("Start parsing variants from stdin")
        variant_file_handle = get_vcf_handle(
            fsock=sys.stdin, 
        )
    else:
        logger.info("Start parsing variants from stdin")
        variant_file_handle = get_vcf_handle(
            infile=variant_file, 
        )
    
    header = []
    nr_of_deleted = 0
    
    start_deleting = datetime.now()
    
    for line in variant_file_handle:
        line = line.rstrip()
        if line.startswith('#'):
            if not line.startswith('##'):
                header = line[1:].split()
        else:
            formated_variant = get_formated_variant(
                variant_line = line,
                header_line = header,
                affected_individuals = affected_individuals
            )
            
            adapter.delete_variant(formated_variant)
            nr_of_deleted += 1
    
    logger.info("Nr of variants deleted: {0}".format(nr_of_deleted))
    logger.info("Time to delete variants: {0}".format(datetime.now() - start_deleting))
    
    
    



import logging

from loqusdb.plugins import BaseCaseMixin
from loqusdb.exceptions import CaseError

logger = logging.getLogger(__name__)

class CaseMixin(BaseCaseMixin):
    
    def case(self, case):
        """Get a case from the database
    
            Search the cases with the case id
        
            Args:
                case (dict): A case dictionary
        
            Returns:
                mongo_case (dict): A mongo case dictionary
        """
        logger.debug("Getting case {0} from database".format(
            case.get('case_id')))
        case_id = case['case_id']
        return self.db.case.find_one({'case_id': case_id})
    
    def add_case(self, case):
        """Add a case to the case collection
        
            If the case exists raise error.
        
            Args:
                db (MongoClient): A connection to the mongodb
                case (dict): A case dictionary
        
        """
        logger.debug("Checking if case {0} exists in database".format(
            case.get('case_id')
        ))
        if self.case(case):
            raise CaseError("Case {0} already exists in database."\
            " Can not add case twice.".format(
                case.get('case_id')
            ))
    
        mongo_case_id = self.db.case.insert_one(case).inserted_id
    
        return mongo_case_id
        
    def delete_case(self, case):
        """Delete case from the database
    
            Delete a case from the database
        
            Args:
                case (dict): A case dictionary
        
        """
        mongo_case = self.case(case)
    
        if mongo_case:
            logger.info("Removing case {0} from database".format(
                mongo_case.get('case_id')
            ))
            self.db.case.remove({'_id': mongo_case['_id']})
        else:
            raise CaseError("Tried to delete case {0} but could not find case".format(
                case.get('case_id')
            ))
        return
    


import os
import logging
import click

from datetime import datetime

from vcftoolbox import get_vcf_handle

from loqusdb.exceptions import CaseError
from loqusdb.vcf_tools import get_formated_variant
from loqusdb.utils import get_family

from . import base_command

logger = logging.getLogger(__name__)

@base_command.command()
@click.argument('variant_file',
                    nargs=1,
                    type=click.Path(),
                    metavar='<vcf_file> or -'
)
@click.option('-f', '--family_file',
                    nargs=1, 
                    type=click.File('r'),
                    metavar='<ped_file>'
)
@click.option('-t' ,'--family_type', 
                type=click.Choice(['ped', 'alt', 'cmms', 'mip']), 
                default='ped',
                help='If the analysis use one of the known setups, please specify which one.'
)
@click.option('-b' ,'--bulk_insert', 
                is_flag=True, 
                help='Insert bulks of variants for better performance'
)
@click.pass_context
def load(ctx, variant_file, family_file, family_type, bulk_insert):
    """Load the variant frequency database
        
        The loading is based on if the variant is seen in a ny affected individual
        in the family.
    """
    if not family_file:
        logger.error("Please provide a family file")
        ctx.abort()
    
    try:
        family = get_family(
            family_lines = family_file, 
            family_type = family_type
        )
    except SyntaxError as err:
        logger.warning(err.message)
        ctx.abort()
    
    family_id = family.family_id
    
    affected_individuals = family.affected_individuals
    
    if not affected_individuals:
        logger.error("No affected individuals could be found in ped file")
        ctx.abort()

    logger.info("Found affected individuals in ped file: {0}".format(
        ', '.join(affected_individuals)
    ))
    
    adapter = ctx.obj['adapter']
    
    case = {
        'case_id': family_id,
        'vcf_path': os.path.abspath(variant_file)
    }
    
    try:
        adapter.add_case(case)
    except CaseError as e:
        logger.error(e.message)
        ctx.abort()
    
    if variant_file == '-':
        logger.info("Start parsing variants from stdin")
        variant_file = get_vcf_handle(
            fsock=sys.stdin, 
        )
    else:
        logger.info("Start parsing variants from stdin")
        variant_file = get_vcf_handle(
            infile=variant_file, 
        )
    
    
    #This is the header line with mandatory vcf fields
    header = []
    nr_of_variants = 0
    nr_of_inserted = 0
    
    start_inserting = datetime.now()
    start_ten_thousand = datetime.now()
    
    variants = []
    for line in variant_file:
        line = line.rstrip()
        if line.startswith('#'):
            if not line.startswith('##'):
                header = line[1:].split()
        else:
            nr_of_variants += 1
            
            formated_variant = get_formated_variant(
                variant_line = line,
                header_line = header,
                affected_individuals = affected_individuals
            )
            
            if formated_variant:
                nr_of_inserted += 1
                if bulk_insert:
                    variants.append(formated_variant)
                else:
                    adapter.add_variant(variant=formated_variant)
            
            if nr_of_variants % 10000 == 0:
                logger.info("{0} of variants processed".format(nr_of_variants))
                logger.info("Time to insert last 10000: {0}".format(
                    datetime.now()-start_ten_thousand))
                start_ten_thousand = datetime.now()
                
            if nr_of_variants % 100000 == 0:
                if bulk_insert:
                    adapter.add_bulk(variants)
                    variants = []
    
    if bulk_insert:
        adapter.add_bulk(variants)
    
    logger.info("Nr of variants in vcf: {0}".format(nr_of_variants))
    logger.info("Nr of variants inserted: {0}".format(nr_of_inserted))
    logger.info("Time to insert variants: {0}".format(datetime.now() - start_inserting))
    
    



import logging

from vcftoolbox import Genotype

logger = logging.getLogger(__name__)

def get_variant_id(chrom, pos, ref, alt):
    """docstring for get_variant_id"""
    return '_'.join([chrom, pos, ref, alt])

def get_formated_variant(variant_line=None, variant_dict=None, header_line = None,
                        affected_individuals = None):
    """Return a formatted variant line
    
        Take a vcf formatted variant line and return a dictionary with the
        relevant information.
        
        Args:
            variant_line (str): A vcf formatted variant line
            variant_dict (dict): A variant dictionary
        
        Return:
            formatted_variant (dict): A variant dictionary
    """
    if not header_line:
        raise Exception("Need to provide a header line to format variant")
    
    formatted_variant = {}
    
    if variant_line:
        splitted_line = variant_line.rstrip().split('\t')
        
        chrom = splitted_line[0].rstrip('chr')
        pos = splitted_line[1]
        ref = splitted_line[3]
        alt = splitted_line[4]
        
        if len(splitted_line) > 7:
            format_field = splitted_line[8].split(':')
        
    
    elif variant_dict:
        chrom = variant_dict['CHROM'].rstrip('chr')
        pos = variant_dict['POS']
        ref = variant_dict['REF']
        alt = variant_dict['ALT']
        
        format_field = variant_dict.get('FORMAT','').split(':')
        

    if ',' in alt:
        raise Exception("Multi allele calls are not allowed.")

    
    found_variant = False
    found_homozygote = False
    
    
    for index, ind_id in enumerate(header_line):
        if ind_id in affected_individuals:
            if variant_line:
                raw_gt_call = splitted_line[index].split(':')
            elif variant_dict:
                raw_gt_call = variant_dict.get(ind_id,'').split(':')
                
            gt_call = dict(zip(
                    format_field,
                    raw_gt_call)
                    )
            
            genotype = Genotype(**gt_call)
            
            if genotype.has_variant:
                logger.debug("Found variant in affected")
                found_variant = True
            if genotype.homo_alt:
                logger.debug("Found homozygote alternative variant in affected")
                found_homozygote = True
    
    
    if found_variant:
        formatted_variant['_id'] = get_variant_id(
            chrom = chrom, 
            pos = pos, 
            ref = ref, 
            alt = alt
        )
        if found_homozygote:
            formatted_variant['homozygote'] = 1
        else:
            formatted_variant['homozygote'] = 0
            
    
    return formatted_variant
